
Class AerialCamera

	Public
	
		Property CameraDistance:Float ()
			Return camera_distance
			Setter (distance:Float)
				camera_distance = distance
		End
		
		Property RenderCam:Camera ()
			Return real_camera
			Setter (new_cam:Camera)
				real_camera = new_cam
		End
		
		Method New (viewport:Recti, current_camera:AerialCamera, range_far:Float)
			
			If current_camera Then current_camera.Destroy ()
			
			camera_pivot					= New Pivot
			
			RenderCam						= New Camera (camera_pivot)
			RenderCam.Near					= 0.01
			RenderCam.Far					= range_far * Sqrt (3.0) ' Terrain cube diagonal
			RenderCam.FOV					= 90.0 ' Mojo3d default
			
			RenderCam.Viewport				= viewport
	
			' Chase target -- position camera tries to move towards...
			
			chase_target 					= Model.CreateSphere (1, 32, 32, New PbrMaterial (Color.Red))
			chase_target.Alpha				= 0.5
			chase_target.Material.CullMode	= CullMode.None
			chase_target.Visible			= False
			
			up = New Vec3f (0.0, up_y_default, 0.0)
			
			Reset ()
			
		End
		
		Method Destroy ()
		
			camera_pivot?.Destroy ()
			real_camera?.Destroy ()
			chase_target?.Destroy ()
			
		End
		
		Method Update (target:PlaneBehaviour)
	
			prevvel = lastvel
			
			Local plane_body:RigidBody = target.Entity.GetComponent <RigidBody> ()
			Local plane_model:Model = Cast <Model> (target.Entity)
			
			lastvel = lastvel.Blend (plane_body.LinearVelocity, 0.045)

			chase_target.Position = (plane_model.Position + up) - (lastvel * CameraDistance)

			camera_pivot.Move ((chase_target.Position - camera_pivot.Position) * 0.95, True)
			camera_pivot.PointAt (plane_model)

			Local cam_dist:Float = plane_model.Position.Distance (RenderCam.Position)
			
			Local closeup:Float = 10.5
			Local closer:Float = 0.1
			
			If cam_dist < closeup
				RenderCam.FOV = Blend (RenderCam.FOV, TransformRange (cam_dist, 1.0, closeup, 130.0, 90.0), closer)
				up.Y = Blend (up.Y, 3.0, 0.01)
			Else
				RenderCam.FOV = Blend (RenderCam.FOV, 90.0, 0.075)
				up.Y = Blend (up.Y, up_y_default, 0.01)
			Endif

		End

		Method Render (canvas:Canvas)
			RenderCam.Render (canvas)
		End
		
		Method Move (tv:Vec3f, localSpace:Bool = False)
			camera_pivot.Move (tv, localSpace)
		End
		
		Method Move (tx:Float, ty:Float, tz:Float)
			camera_pivot.Move (tx, ty, tz)
		End
		
		Method PointAt (target:Entity)
			camera_pivot.PointAt (target)
		End
		
		Method Position (v3:Vec3f)
			camera_pivot.Position = v3
		End

		Method Reset ()
	
			RenderCam.FOV	= 90.0
	
			lastvel			= New Vec3f (0, 0, 15)
			prevvel			= lastvel
	
		End
		
	Private
	
		Field camera_pivot:Pivot
		Field real_camera:Camera
		
		Field chase_target:Model
	
		Field lastvel:Vec3f
		Field prevvel:Vec3f
	
		Field up:Vec3f
		Field up_y_default:Float = 2.5
		
		Field camera_distance:Float = 0.4
		
End
